@@ -673,7 +673,9 @@ function OutputStream(options) {
         }
     }
     PARENS(AST_AsyncFunction, needs_parens_function);
+    PARENS(AST_AsyncGeneratorFunction, needs_parens_function);
     PARENS(AST_Function, needs_parens_function);
+    PARENS(AST_GeneratorFunction, needs_parens_function);
 
     // same goes for an object literal, because otherwise it would be
     // interpreted as a block of code.
@@ -682,14 +684,19 @@ function OutputStream(options) {
     }
     PARENS(AST_Object, needs_parens_obj);
 
-    PARENS(AST_Unary, function(output) {
+    function needs_parens_unary(output) {
         var p = output.parent();
         // (-x) ** y
         if (p instanceof AST_Binary) return p.operator == "**" && p.left === this;
-        // (x++).toString(3)
-        // (typeof x).length
-        return (p instanceof AST_Call || p instanceof AST_PropAccess) && p.expression === this;
-    });
+        // (await x)(y)
+        // new (await x)
+        if (p instanceof AST_Call) return p.expression === this;
+        // (x++)[y]
+        // (typeof x).y
+        if (p instanceof AST_PropAccess) return p.expression === this;
+    }
+    PARENS(AST_Await, needs_parens_unary);
+    PARENS(AST_Unary, needs_parens_unary);
 
     PARENS(AST_Sequence, function(output) {
         var p = output.parent();
@@ -708,9 +715,13 @@ function OutputStream(options) {
             || p instanceof AST_Conditional
             // [ a = (1, 2) ] = [] ---> a == 2
             || p instanceof AST_DefaultValue
+            // { [(1, 2)]: foo } = bar
+            // { 1: (2, foo) } = bar
+            || p instanceof AST_DestructuredKeyVal
+            // for (foo of (bar, baz));
+            || p instanceof AST_ForOf
             // { [(1, 2)]: 3 }[2] ---> 3
             // { foo: (1, 2) }.foo ---> 2
-            || p instanceof AST_DestructuredKeyVal
             || p instanceof AST_ObjectProperty
             // (1, {foo:2}).foo or (1, {foo:2})["foo"] ---> 2
             || p instanceof AST_PropAccess && p.expression === this
@@ -719,7 +730,9 @@ function OutputStream(options) {
             // !(foo, bar, baz)
             || p instanceof AST_Unary
             // var a = (1, 2), b = a + a; ---> b == 4
-            || p instanceof AST_VarDef;
+            || p instanceof AST_VarDef
+            // yield (foo, bar)
+            || p instanceof AST_Yield;
     });
 
     PARENS(AST_Binary, function(output) {
@@ -826,17 +839,8 @@ function OutputStream(options) {
     PARENS(AST_Conditional, function(output) {
         return needs_parens_assign_cond(this, output);
     });
-
-    PARENS(AST_Await, function(output) {
-        var p = output.parent();
-        // (await x) ** y
-        if (p instanceof AST_Binary) return p.operator == "**" && p.left === this;
-        // new (await foo)
-        // (await foo)(bar)
-        if (p instanceof AST_Call) return p.expression === this;
-        // (await foo).prop
-        // (await foo)["prop"]
-        if (p instanceof AST_PropAccess) return p.expression === this;
+    PARENS(AST_Yield, function(output) {
+        return needs_parens_assign_cond(this, output);
     });
 
     /* -----[ PRINTERS ]----- */
@@ -978,20 +982,25 @@ function OutputStream(options) {
         output.space();
         force_statement(self.body, output);
     });
-    DEFPRINT(AST_ForIn, function(output) {
-        var self = this;
-        output.print("for");
-        output.space();
-        output.with_parens(function() {
-            self.init.print(output);
+    function print_for_enum(prefix, infix) {
+        return function(output) {
+            var self = this;
+            output.print(prefix);
             output.space();
-            output.print("in");
+            output.with_parens(function() {
+                self.init.print(output);
+                output.space();
+                output.print(infix);
+                output.space();
+                self.object.print(output);
+            });
             output.space();
-            self.object.print(output);
-        });
-        output.space();
-        force_statement(self.body, output);
-    });
+            force_statement(self.body, output);
+        };
+    }
+    DEFPRINT(AST_ForAwaitOf, print_for_enum("for await", "of"));
+    DEFPRINT(AST_ForIn, print_for_enum("for", "in"));
+    DEFPRINT(AST_ForOf, print_for_enum("for", "of"));
     DEFPRINT(AST_With, function(output) {
         var self = this;
         output.print("with");
@@ -1061,6 +1070,20 @@ function OutputStream(options) {
     }
     DEFPRINT(AST_AsyncDefun, print_async);
     DEFPRINT(AST_AsyncFunction, print_async);
+    function print_async_generator(output) {
+        output.print("async");
+        output.space();
+        output.print("function*");
+        print_lambda(this, output);
+    }
+    DEFPRINT(AST_AsyncGeneratorDefun, print_async_generator);
+    DEFPRINT(AST_AsyncGeneratorFunction, print_async_generator);
+    function print_generator(output) {
+        output.print("function*");
+        print_lambda(this, output);
+    }
+    DEFPRINT(AST_GeneratorDefun, print_generator);
+    DEFPRINT(AST_GeneratorFunction, print_generator);
 
     /* -----[ jumps ]----- */
     function print_jump(kind, prop) {
@@ -1212,7 +1235,7 @@ function OutputStream(options) {
                 def.print(output);
             });
             var p = output.parent();
-            if (p && p.init !== self || !(p instanceof AST_For || p instanceof AST_ForIn)) output.semicolon();
+            if (!(p instanceof AST_IterationStatement && p.init === self)) output.semicolon();
         };
     }
     DEFPRINT(AST_Const, print_definitinos("const"));
@@ -1239,7 +1262,7 @@ function OutputStream(options) {
             output.print("=");
             output.space();
             var p = output.parent(1);
-            var noin = p instanceof AST_For || p instanceof AST_ForIn;
+            var noin = p instanceof AST_For || p instanceof AST_ForEnumeration;
             parenthesize_for_noin(self.value, output, noin);
         }
     });
@@ -1360,6 +1383,13 @@ function OutputStream(options) {
         output.space();
         this.expression.print(output);
     });
+    DEFPRINT(AST_Yield, function(output) {
+        output.print(this.nested ? "yield*" : "yield");
+        if (this.expression) {
+            output.space();
+            this.expression.print(output);
+        }
+    });
 
     /* -----[ literals ]----- */
     DEFPRINT(AST_Array, function(output) {
