@@ -4,15 +4,16 @@ unused_funarg_1: {
         unused: true,
     }
     input: {
-        function f(a, b, c, d, e) {
+        console.log(function f(a, b, c, d, e) {
             return a + b;
-        }
+        }(14, 28));
     }
     expect: {
-        function f(a, b) {
+        console.log(function(a, b) {
             return a + b;
-        }
+        }(14, 28));
     }
+    expect_stdout: "42"
 }
 
 unused_funarg_2: {
@@ -21,15 +22,16 @@ unused_funarg_2: {
         unused: true,
     }
     input: {
-        function f(a, b, c, d, e) {
+        console.log(function f(a, b, c, d, e) {
             return a + c;
-        }
+        }(14, 21, 28));
     }
     expect: {
-        function f(a, b, c) {
+        console.log(function(a, c) {
             return a + c;
-        }
+        }(14, 28));
     }
+    expect_stdout: "42"
 }
 
 unused_nested_function: {
@@ -357,37 +359,6 @@ drop_toplevel_vars: {
     }
 }
 
-drop_toplevel_vars_fargs: {
-    options = {
-        keep_fargs: false,
-        toplevel: "vars",
-        unused: true,
-    }
-    input: {
-        var a, b = 1, c = g;
-        function f(d) {
-            return function() {
-                c = 2;
-            };
-        }
-        a = 2;
-        function g() {}
-        function h() {}
-        console.log(b = 3);
-    }
-    expect: {
-        function f() {
-            return function() {
-                2;
-            };
-        }
-        2;
-        function g() {}
-        function h() {}
-        console.log(3);
-    }
-}
-
 drop_toplevel_all: {
     options = {
         toplevel: true,
@@ -625,13 +596,14 @@ drop_fargs: {
         unused: true,
     }
     input: {
-        function f(a) {
+        console.log(function f(a) {
             var b = a;
-        }
+        }());
     }
     expect: {
-        function f() {}
+        console.log(function() {}());
     }
+    expect_stdout: "undefined"
 }
 
 drop_fnames: {
@@ -1730,7 +1702,7 @@ chained_3: {
     expect: {
         console.log(function(a, b) {
             var c = b;
-            b++;
+            +b;
             return c;
         }(0, 2));
     }
@@ -1997,7 +1969,7 @@ issue_3146_4: {
     expect_stdout: "PASS"
 }
 
-issue_3192: {
+issue_3192_1: {
     options = {
         unused: true,
     }
@@ -2025,6 +1997,26 @@ issue_3192: {
     ]
 }
 
+issue_3192_2: {
+    options = {
+        keep_fargs: false,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        (function(a) {
+            console.log(a = "foo", arguments[0]);
+        })("bar");
+    }
+    expect: {
+        "use strict";
+        (function() {
+            console.log("foo", arguments[0]);
+        })("bar");
+    }
+    expect_stdout: "foo bar"
+}
+
 issue_3233: {
     options = {
         pure_getters: "strict",
@@ -2146,6 +2138,7 @@ issue_3497: {
 issue_3515_1: {
     options = {
         collapse_vars: true,
+        evaluate: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2161,8 +2154,7 @@ issue_3515_1: {
     expect: {
         var c = 0;
         (function() {
-            this[c++] = 0;
-            for (var key20 in !0);
+            for (var key20 in !(this[c++] = 0));
         })();
         console.log(c);
     }
@@ -2198,6 +2190,7 @@ issue_3515_2: {
 issue_3515_3: {
     options = {
         collapse_vars: true,
+        evaluate: true,
         unused: true,
     }
     input: {
@@ -2265,6 +2258,7 @@ function_assign: {
 issue_3598: {
     options = {
         collapse_vars: true,
+        evaluate: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2416,7 +2410,7 @@ issue_3673: {
 
 issue_3746: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         side_effects: true,
         unused: true,
     }
@@ -2678,8 +2672,7 @@ issue_3956: {
         })();
     }
     expect: {
-        var c, d;
-        c += 0,
+        var d;
         console.log(NaN),
         d = 1 ^ console.log(1),
         console.log(d);
@@ -2694,7 +2687,7 @@ issue_3956: {
 issue_3962_1: {
     options = {
         evaluate: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_vars: true,
         toplevel: true,
         unused: true,
@@ -2712,13 +2705,13 @@ issue_3962_1: {
     }
     expect: {
         var a = 0;
-        a = (function(c) {
+        (function(c) {
             do {
                 console;
                 0..toString();
             } while (0);
             if (c) console.log("PASS");
-        }((a--, 1)), 0);
+        })(1);
         void 0;
     }
     expect_stdout: "PASS"
@@ -2726,7 +2719,7 @@ issue_3962_1: {
 
 issue_3962_2: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_vars: true,
         side_effects: true,
         toplevel: true,
@@ -2745,13 +2738,13 @@ issue_3962_2: {
     }
     expect: {
         var a = 0;
-        a = (function(c) {
+        (function(c) {
             do {
                 console;
                 0..toString();
             } while (0);
             if (c) console.log("PASS");
-        }((a--, 1)), 0);
+        })(1);
     }
     expect_stdout: "PASS"
 }
@@ -2808,7 +2801,9 @@ issue_4017: {
         var a = 0;
         console.log(function() {
             c &= 0;
-            var c = (a++, A = a, 0);
+            var c;
+            a++,
+            A = a;
         }());
     }
     expect_stdout: "undefined"
@@ -2834,11 +2829,11 @@ issue_4025: {
         console.log(a, b, d);
     }
     expect: {
-        var d, c = 0;
+        var c = 0;
         try {
             console.log(c);
         } finally {
-            d = c + 1;
+            var d = c + 1;
             c = 0;
         }
         console.log(1, 1, d);
@@ -2848,3 +2843,411 @@ issue_4025: {
         "1 1 1",
     ]
 }
+
+forin_var_1: {
+    options = {
+        unused: true,
+    }
+    input: {
+        var k;
+        for (k in [ 1, 2 ])
+            console.log(k);
+        for (k in { PASS: 3 })
+            console.log(k);
+        console.log(k);
+    }
+    expect: {
+        for (var k in [ 1, 2 ])
+            console.log(k);
+        for (k in { PASS: 3 })
+            console.log(k);
+        console.log(k);
+    }
+    expect_stdout: [
+        "0",
+        "1",
+        "PASS",
+        "PASS",
+    ]
+}
+
+forin_var_2: {
+    options = {
+        unused: true,
+    }
+    input: {
+        console.log(function() {
+            switch (0) {
+              case function() {
+                    for (a in 0);
+                }:
+                var b = 0;
+            }
+            for (var c = 0; a;);
+            var a;
+        }());
+    }
+    expect: {
+        console.log(function() {
+            switch (0) {
+              case function() {
+                    for (a in 0);
+                }:
+            }
+            for (; a;);
+            var a;
+        }());
+    }
+    expect_stdout: "undefined"
+}
+
+issue_4133: {
+    options = {
+        evaluate: true,
+        merge_vars: true,
+        pure_getters: "strict",
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 1;
+        var b = [ a-- ], c = b && b[c];
+        console.log(a);
+    }
+    expect: {
+        var b = 1;
+        console.log(0);
+    }
+    expect_stdout: "0"
+}
+
+issue_4144: {
+    options = {
+        keep_fargs: false,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function(a, b) {
+            var b = console, c = ++b;
+        })(console.log("PASS"), 0);
+    }
+    expect: {
+        (function(b) {
+            b = console,
+            ++b;
+        })(console.log("PASS"));
+    }
+    expect_stdout: "PASS"
+}
+
+issue_4146: {
+    options = {
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function f(a, b) {
+            function g() {}
+            var a = g;
+            var c = b;
+            c.p;
+            console.log(typeof a);
+        }
+        f("FAIL", 42);
+    }
+    expect: {
+        (function(a, b) {
+            a = function () {};
+            var c = b;
+            c.p;
+            console.log(typeof a);
+        })(0, 42);
+    }
+    expect_stdout: "function"
+}
+
+var_catch_redefined: {
+    options = {
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = "FAIL";
+        try {
+            throw "PASS";
+        } catch (a) {
+            function f() {
+                return a;
+            }
+            console.log(a);
+        }
+        f();
+    }
+    expect: {
+        var a = "FAIL";
+        try {
+            throw "PASS";
+        } catch (a) {
+            function f() {
+                return a;
+            }
+            console.log(a);
+        }
+        f();
+    }
+    expect_stdout: "PASS"
+}
+
+single_use_catch_redefined: {
+    options = {
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 1;
+        try {
+            throw 2;
+        } catch (a) {
+            function g() {
+                return a;
+            }
+        }
+        console.log(g());
+    }
+    expect: {
+        var a = 1;
+        try {
+            throw 2;
+        } catch (a) {
+            function g() {
+                return a;
+            }
+        }
+        console.log(g());
+    }
+    expect_stdout: true
+}
+
+issue_4184: {
+    options = {
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function() {
+            var a = function() {}, b = [ a, 1 && b, a = {} ];
+            try {
+                throw 42;
+            } catch (a) {
+                {
+                    console.log(a);
+                }
+            }
+        })();
+    }
+    expect: {
+        (function() {
+            var b = [ function() {}, 1 && b, {} ];
+            try {
+                throw 42;
+            } catch (a) {
+                console.log(a);
+            }
+        })();
+    }
+    expect_stdout: "42"
+}
+
+issue_4235: {
+    options = {
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+        varify: true,
+    }
+    input: {
+        (function() {
+            {
+                const f = 0;
+            }
+            (function f() {
+                var f = console.log(f);
+            })();
+        })();
+    }
+    expect: {
+        (function() {
+            f = console.log(f),
+            void 0;
+            var f;
+        })();
+    }
+    expect_stdout: "undefined"
+}
+
+issue_4404: {
+    options = {
+        pure_getters: "strict",
+        unused: true,
+    }
+    input: {
+        function f(a) {
+            arguments[0] = "PASS";
+            console.log(a);
+        }
+        f("FAIL");
+    }
+    expect: {
+        function f(a) {
+            arguments[0] = "PASS";
+            console.log(a);
+        }
+        f("FAIL");
+    }
+    expect_stdout: "PASS"
+}
+
+issue_4413: {
+    options = {
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        console.log(function f(arguments) {
+            var arguments = function() {};
+            return arguments.length;
+        }());
+    }
+    expect: {
+        console.log(function(arguments) {
+            return function() {}.length;
+        }());
+    }
+    expect_stdout: "0"
+}
+
+issue_4464_1: {
+    options = {
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        function f(a) {
+            var a = function() {};
+            return [ arguments, a ];
+        }
+        console.log(typeof f()[1]);
+    }
+    expect: {
+        function f(a) {
+            a = function() {};
+            return [ arguments, a ];
+        }
+        console.log(typeof f()[1]);
+    }
+    expect_stdout: "function"
+}
+
+issue_4464_2: {
+    options = {
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        function f(a) {
+            var a = function() {};
+            return [ arguments, a ];
+        }
+        console.log(typeof f(42)[0][0]);
+    }
+    expect: {
+        function f(a) {
+            a = function() {};
+            return [ arguments, a ];
+        }
+        console.log(typeof f(42)[0][0]);
+    }
+    expect_stdout: "function"
+}
+
+issue_4464_3: {
+    options = {
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function a(a) {
+            var a = function() {};
+            return [ arguments[0], a ];
+        })(42).forEach(function(b) {
+            console.log(typeof b);
+        });
+    }
+    expect: {
+        (function(a) {
+            a = function() {};
+            return [ arguments[0], a ];
+        })(42).forEach(function(b) {
+            console.log(typeof b);
+        });
+    }
+    expect_stdout: [
+        "function",
+        "function",
+    ]
+}
+
+issue_4558_1: {
+    options = {
+        evaluate: true,
+        pure_getters: true,
+        reduce_vars: true,
+        sequences: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 0;
+        var b = 1, b = c >>>= a;
+        var c = 0;
+        b && 0[a++],
+        console.log(a);
+    }
+    expect: {
+        var a = 0;
+        var b = c >>>= a;
+        var c;
+        b && a++,
+        console.log(a);
+    }
+    expect_stdout: "0"
+}
+
+issue_4558_2: {
+    options = {
+        evaluate: true,
+        ie8: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function() {
+            var a = 1;
+            var b = (a = NaN) || (console.log("PASS"), 2);
+            return a;
+        })();
+    }
+    expect: {
+        (function() {
+            var a;
+            (a = NaN) || console.log("PASS");
+            return a;
+        })();
+    }
+    expect_stdout: "PASS"
+}
