@@ -18,43 +18,6 @@ keep_fargs_false: {
         function j(e) {}
         console.log(h(), i().length, j.length);
     }
-    expect: {
-        console.log(function f() {
-            return f.length;
-        }(), function g() {
-            return g;
-        }().length);
-        function h() {
-            return h.length;
-        }
-        function i() {
-            return i;
-        }
-        function j() {}
-        console.log(h(), i().length, j.length);
-    }
-}
-
-keep_fargs_strict: {
-    options = {
-        keep_fargs: "strict",
-        unused: true,
-    }
-    input: {
-        console.log(function f(a) {
-            return f.length;
-        }(), function g(b) {
-            return g;
-        }().length);
-        function h(c) {
-            return h.length;
-        }
-        function i(d) {
-            return i;
-        }
-        function j(e) {}
-        console.log(h(), i().length, j.length);
-    }
     expect: {
         console.log(function f(a) {
             return f.length;
@@ -117,61 +80,11 @@ keep_fargs_true: {
     ]
 }
 
-replace_index: {
-    options = {
-        arguments: true,
-        evaluate: true,
-        keep_fargs: "strict",
-        properties: true,
-    }
-    input: {
-        var arguments = [];
-        console.log(arguments[0]);
-        (function() {
-            console.log(arguments[1], arguments["1"], arguments["foo"]);
-        })("bar", 42);
-        (function(a, b) {
-            console.log(arguments[1], arguments["1"], arguments["foo"]);
-        })("bar", 42);
-        (function(arguments) {
-            console.log(arguments[1], arguments["1"], arguments["foo"]);
-        })("bar", 42);
-        (function() {
-            var arguments;
-            console.log(arguments[1], arguments["1"], arguments["foo"]);
-        })("bar", 42);
-    }
-    expect: {
-        var arguments = [];
-        console.log(arguments[0]);
-        (function(argument_0, argument_1) {
-            console.log(argument_1, argument_1, arguments.foo);
-        })("bar", 42);
-        (function(a, b) {
-            console.log(b, b, arguments.foo);
-        })("bar", 42);
-        (function(arguments) {
-            console.log(arguments[1], arguments[1], arguments.foo);
-        })("bar", 42);
-        (function() {
-            var arguments;
-            console.log(arguments[1], arguments[1], arguments.foo);
-        })("bar", 42);
-    }
-    expect_stdout: [
-        "undefined",
-        "42 42 undefined",
-        "42 42 undefined",
-        "a a undefined",
-        "42 42 undefined",
-    ]
-}
-
 replace_index_strict: {
     options = {
         arguments: true,
         evaluate: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         properties: true,
         reduce_vars: true,
     }
@@ -202,7 +115,7 @@ replace_index_strict: {
 issue_1858: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         pure_getters: true,
         unused: true,
     }
@@ -224,7 +137,7 @@ issue_1858: {
 issue_2187_2: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -245,7 +158,7 @@ issue_2187_2: {
 issue_2203_2: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -280,7 +193,7 @@ issue_2203_2: {
 issue_2298: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         passes: 2,
         reduce_funcs: true,
         reduce_vars: true,
@@ -306,7 +219,6 @@ issue_2298: {
     expect: {
         !function() {
             (function() {
-                0;
                 try {
                     !function() {
                         (void 0)[1] = "foo";
@@ -324,7 +236,7 @@ issue_2298: {
 issue_2319_1: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -347,7 +259,7 @@ issue_2319_1: {
 issue_2319_2: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -372,7 +284,7 @@ issue_2319_2: {
 issue_2319_3: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -397,7 +309,7 @@ issue_2319_3: {
 issue_2425_1: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -420,7 +332,7 @@ issue_2425_1: {
 issue_2425_2: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -443,7 +355,7 @@ issue_2425_2: {
 issue_2425_3: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -466,7 +378,7 @@ issue_2425_3: {
 issue_2436_13: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         passes: 2,
         reduce_vars: true,
         unused: true,
@@ -500,7 +412,7 @@ issue_2436_13: {
 issue_2506: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         passes: 2,
         reduce_vars: true,
         unused: true,
@@ -526,7 +438,7 @@ issue_2506: {
         function f0(bar) {
             (function() {
                 (function() {
-                    if (false <= 0/0 & this >> 1 >= 0)
+                    if (false <= NaN & this >> 1 >= 0)
                         c++;
                 })(c++);
             })();
@@ -539,7 +451,7 @@ issue_2506: {
 
 issue_2226_1: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         side_effects: true,
         unused: true,
     }
@@ -586,7 +498,7 @@ issue_2226_1: {
 issue_2226_2: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         sequences: true,
         side_effects: true,
         unused: true,
@@ -608,7 +520,7 @@ issue_2226_2: {
 issue_2226_3: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         side_effects: true,
         unused: true,
     }
@@ -628,7 +540,7 @@ issue_2226_3: {
 
 issue_3192: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -658,7 +570,7 @@ issue_3192: {
 if_increment: {
     options = {
         evaluate: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_vars: true,
         unused: true,
     }
@@ -680,7 +592,7 @@ if_increment: {
 try_increment: {
     options = {
         evaluate: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_vars: true,
         unused: true,
     }
@@ -704,7 +616,7 @@ try_increment: {
 issue_2630_3: {
     options = {
         inline: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_vars: true,
         unused: true,
     }
@@ -741,7 +653,7 @@ issue_2630_3: {
 issue_3364: {
     options = {
         functions: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_vars: true,
         toplevel: true,
         unused: true,
@@ -806,7 +718,7 @@ issue_3364: {
 
 defun_label: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         passes: 2,
         reduce_funcs: true,
         reduce_vars: true,
@@ -838,7 +750,7 @@ defun_label: {
 
 iife_func_side_effects: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_funcs: true,
         reduce_vars: true,
         unused: true,
@@ -890,7 +802,7 @@ iife_func_side_effects: {
 issue_1595_1: {
     options = {
         evaluate: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_funcs: true,
         reduce_vars: true,
         unused: true,
@@ -910,7 +822,7 @@ issue_1595_1: {
 issue_1595_2: {
     options = {
         evaluate: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_funcs: true,
         reduce_vars: true,
         unused: true,
@@ -930,7 +842,7 @@ issue_1595_2: {
 issue_1595_3: {
     options = {
         evaluate: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         passes: 2,
         reduce_funcs: true,
         reduce_vars: true,
@@ -951,7 +863,7 @@ issue_1595_3: {
 issue_1595_4: {
     options = {
         evaluate: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_funcs: true,
         reduce_vars: true,
         unused: true,
@@ -973,7 +885,7 @@ issue_1595_4: {
 
 duplicate_lambda_defun_name_1: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_vars: true,
     }
     input: {
@@ -993,7 +905,7 @@ duplicate_lambda_defun_name_1: {
 
 duplicate_lambda_defun_name_2: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         passes: 2,
         reduce_vars: true,
         unused: true,
@@ -1014,7 +926,7 @@ duplicate_lambda_defun_name_2: {
 
 function_name_mangle: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         keep_fnames: true,
         reduce_vars: true,
         unused: true,
@@ -1032,7 +944,7 @@ function_name_mangle: {
 
 function_name_mangle_ie8: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         keep_fnames: true,
         reduce_vars: true,
         unused: true,
@@ -1053,7 +965,7 @@ function_name_mangle_ie8: {
 
 issue_3420_1: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -1076,7 +988,7 @@ issue_3420_1: {
 issue_3420_2: {
     options = {
         inline: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -1097,7 +1009,7 @@ issue_3420_2: {
 issue_3420_3: {
     options = {
         inline: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_vars: true,
         unused: true,
     }
@@ -1119,7 +1031,7 @@ issue_3420_3: {
 
 issue_3423_1: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -1139,7 +1051,7 @@ issue_3423_1: {
 
 issue_3423_2: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -1166,7 +1078,7 @@ collapse_vars_repeated: {
         hoist_funs: true,
         if_return: true,
         join_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         loops: true,
         properties: true,
         reduce_funcs: true,
@@ -1213,7 +1125,7 @@ collapse_vars_repeated: {
 chained_3: {
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -1237,7 +1149,7 @@ replace_all_var_scope: {
     rename = true
     options = {
         collapse_vars: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     mangle = {}
@@ -1266,7 +1178,7 @@ replace_all_var_scope: {
 
 issue_1583: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         passes: 2,
         reduce_funcs: true,
         reduce_vars: true,
@@ -1303,7 +1215,7 @@ issues_3267_1: {
         dead_code: true,
         evaluate: true,
         inline: true,
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_vars: true,
         sequences: true,
         side_effects: true,
@@ -1332,7 +1244,7 @@ issues_3267_1: {
 
 trailing_argument_side_effects: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -1356,7 +1268,7 @@ trailing_argument_side_effects: {
 
 recursive_iife_1: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_vars: true,
         unused: true,
     }
@@ -1375,7 +1287,7 @@ recursive_iife_1: {
 
 recursive_iife_2: {
     options = {
-        keep_fargs: "strict",
+        keep_fargs: false,
         reduce_vars: true,
         unused: true,
     }
@@ -1453,3 +1365,37 @@ issue_3619: {
     }
     expect_stdout: "PASS"
 }
+
+issue_4353_1: {
+    options = {
+        keep_fargs: false,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        console.log(function f(a) {}.length);
+    }
+    expect: {
+        console.log(function(a) {}.length);
+    }
+    expect_stdout: "1"
+}
+
+issue_4353_2: {
+    options = {
+        keep_fargs: false,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function f(a) {
+            while (console.log("PASS"));
+        })();
+    }
+    expect: {
+        (function() {
+            while (console.log("PASS"));
+        })();
+    }
+    expect_stdout: "PASS"
+}
