@@ -1,49 +1,100 @@
-hex_numbers_in_parentheses_for_prototype_functions: {
+parentheses_for_prototype_functions: {
     beautify = {
         beautify: true,
     }
     input: {
-        function f() {
-            (-2);
-            (-2).toFixed(0);
+        (function() {
+            console.log((-2));
+            console.log((-2).toFixed(0));
 
-            (2);
-            (2).toFixed(0);
+            console.log((2));
+            console.log((2).toFixed(0));
 
-            (0.2);
-            (0.2).toFixed(0);
+            console.log((0.2));
+            console.log((0.2).toFixed(0));
 
-            (2.34e20);
-            (2.34e20).toFixed(0);
+            console.log((2.34e20));
+            console.log((2.34e20).toFixed(0));
 
-            (0.00000002);
-            (0.00000002).toFixed(0);
+            console.log((0.00000002));
+            console.log((0.00000002).toFixed(0));
 
-            (1000000000000000128);
-            (1000000000000000128).toFixed(0);
+            console.log((1000000000000000128));
+            console.log((1000000000000000128).toFixed(0));
 
-            (-1000000000000000128);
-            (-1000000000000000128).toFixed(0);
-        }
+            console.log((-1000000000000000128));
+            console.log((-1000000000000000128).toFixed(0));
+        })();
     }
     expect_exact: [
-        "function f() {",
-        "    -2;",
-        "    (-2).toFixed(0);",
-        "    2;",
-        "    2..toFixed(0);",
-        "    .2;",
-        "    .2.toFixed(0);",
-        "    234e18;",
-        "    234e18.toFixed(0);",
-        "    2e-8;",
-        "    2e-8.toFixed(0);",
-        "    0xde0b6b3a7640080;",
-        "    (0xde0b6b3a7640080).toFixed(0);",
-        "    -0xde0b6b3a7640080;",
-        "    (-0xde0b6b3a7640080).toFixed(0);",
-        "}",
+        "(function() {",
+        "    console.log(-2);",
+        "    console.log((-2).toFixed(0));",
+        "    console.log(2);",
+        "    console.log(2..toFixed(0));",
+        "    console.log(.2);",
+        "    console.log(.2.toFixed(0));",
+        "    console.log(234e18);",
+        "    console.log(234e18.toFixed(0));",
+        "    console.log(2e-8);",
+        "    console.log(2e-8.toFixed(0));",
+        "    console.log(0xde0b6b3a7640080);",
+        "    console.log(0xde0b6b3a7640080.toFixed(0));",
+        "    console.log(-0xde0b6b3a7640080);",
+        "    console.log((-0xde0b6b3a7640080).toFixed(0));",
+        "})();",
     ]
+    expect_stdout: true
+}
+
+parentheses_for_prototype_functions_galio: {
+    beautify = {
+        beautify: true,
+        galio: true,
+    }
+    input: {
+        (function() {
+            console.log((-2));
+            console.log((-2).toFixed(0));
+
+            console.log((2));
+            console.log((2).toFixed(0));
+
+            console.log((0.2));
+            console.log((0.2).toFixed(0));
+
+            console.log((2.34e20));
+            console.log((2.34e20).toFixed(0));
+
+            console.log((0.00000002));
+            console.log((0.00000002).toFixed(0));
+
+            console.log((1000000000000000128));
+            console.log((1000000000000000128).toFixed(0));
+
+            console.log((-1000000000000000128));
+            console.log((-1000000000000000128).toFixed(0));
+        })();
+    }
+    expect_exact: [
+        "(function() {",
+        "    console.log(-2);",
+        "    console.log((-2).toFixed(0));",
+        "    console.log(2);",
+        "    console.log(2..toFixed(0));",
+        "    console.log(.2);",
+        "    console.log(.2.toFixed(0));",
+        "    console.log(234e18);",
+        "    console.log(234e18.toFixed(0));",
+        "    console.log(2e-8);",
+        "    console.log(2e-8.toFixed(0));",
+        "    console.log(0xde0b6b3a7640080);",
+        "    console.log((0xde0b6b3a7640080).toFixed(0));",
+        "    console.log(-0xde0b6b3a7640080);",
+        "    console.log((-0xde0b6b3a7640080).toFixed(0));",
+        "})();",
+    ]
+    expect_stdout: true
 }
 
 comparisons: {
@@ -91,7 +142,7 @@ evaluate_1: {
     expect: {
         console.log(
             x + 1 + 2,
-            2 * x,
+            2 * +x,
             +x + 1 + 2,
             1 + x + 2 + 3,
             3 | x,
@@ -130,7 +181,7 @@ evaluate_1_unsafe_math: {
     expect: {
         console.log(
             x + 1 + 2,
-            2 * x,
+            2 * +x,
             +x + 3,
             1 + x + 2 + 3,
             3 | x,
@@ -148,45 +199,52 @@ evaluate_1_unsafe_math: {
 evaluate_2: {
     options = {
         evaluate: true,
+        reduce_vars: true,
         unsafe_math: false,
     }
     input: {
-        var x = "42", y = null;
-        [
-            x + 1 + 2,
-            x * 1 * 2,
-            +x + 1 + 2,
-            1 + x + 2 + 3,
-            1 | x | 2 | 3,
-            1 + x-- + 2 + 3,
-            1 + (x*y + 2) + 3,
-            1 + (2 + x + 3),
-            1 + (2 + ~x + 3),
-            -y + (2 + ~x + 3),
-            1 & (2 & x & 3),
-            1 + (2 + (x |= 0) + 3),
-        ].forEach(function(n) {
-            console.log(typeof n, n);
-        });
+        function f(num) {
+            var x = "" + num, y = null;
+            [
+                x + 1 + 2,
+                x * 1 * 2,
+                +x + 1 + 2,
+                1 + x + 2 + 3,
+                1 | x | 2 | 3,
+                1 + x-- + 2 + 3,
+                1 + (x*y + 2) + 3,
+                1 + (2 + x + 3),
+                1 + (2 + ~x + 3),
+                -y + (2 + ~x + 3),
+                1 & (2 & x & 3),
+                1 + (2 + (x |= 0) + 3),
+            ].forEach(function(n) {
+                console.log(typeof n, n);
+            });
+        }
+        f(42);
     }
     expect: {
-        var x = "42", y = null;
-        [
-            x + 1 + 2,
-            2 * x,
-            +x + 1 + 2,
-            1 + x + 2 + 3,
-            3 | x,
-            1 + x-- + 2 + 3,
-            x*y + 2 + 1 + 3,
-            1 + (2 + x + 3),
-            2 + ~x + 3 + 1,
-            2 + ~x + 3 - y,
-            0 & x,
-            2 + (x |= 0) + 3 + 1,
-        ].forEach(function(n) {
-            console.log(typeof n, n);
-        });
+        function f(num) {
+            var x = "" + num, y = null;
+            [
+                x + "12",
+                2 * x,
+                +x + 1 + 2,
+                1 + x + "23",
+                3 | x,
+                1 + x-- + 2 + 3,
+                x*y + 2 + 1 + 3,
+                2 + x + 3 + 1,
+                2 + ~x + 3 + 1,
+                2 + ~x + 3,
+                0 & x,
+                2 + (x |= 0) + 3 + 1,
+            ].forEach(function(n) {
+                console.log(typeof n, n);
+            });
+        }
+        f(42);
     }
     expect_stdout: [
         "string 4212",
@@ -207,45 +265,52 @@ evaluate_2: {
 evaluate_2_unsafe_math: {
     options = {
         evaluate: true,
+        reduce_vars: true,
         unsafe_math: true,
     }
     input: {
-        var x = "42", y = null;
-        [
-            x + 1 + 2,
-            x * 1 * 2,
-            +x + 1 + 2,
-            1 + x + 2 + 3,
-            1 | x | 2 | 3,
-            1 + x-- + 2 + 3,
-            1 + (x*y + 2) + 3,
-            1 + (2 + x + 3),
-            1 + (2 + ~x + 3),
-            -y + (2 + ~x + 3),
-            1 & (2 & x & 3),
-            1 + (2 + (x |= 0) + 3),
-        ].forEach(function(n) {
-            console.log(typeof n, n);
-        });
+        function f(num) {
+            var x = "" + num, y = null;
+            [
+                x + 1 + 2,
+                x * 1 * 2,
+                +x + 1 + 2,
+                1 + x + 2 + 3,
+                1 | x | 2 | 3,
+                1 + x-- + 2 + 3,
+                1 + (x*y + 2) + 3,
+                1 + (2 + x + 3),
+                1 + (2 + ~x + 3),
+                -y + (2 + ~x + 3),
+                1 & (2 & x & 3),
+                1 + (2 + (x |= 0) + 3),
+            ].forEach(function(n) {
+                console.log(typeof n, n);
+            });
+        }
+        f(42);
     }
     expect: {
-        var x = "42", y = null;
-        [
-            x + 1 + 2,
-            2 * x,
-            +x + 3,
-            1 + x + 2 + 3,
-            3 | x,
-            6 + x--,
-            x*y + 6,
-            1 + (2 + x + 3),
-            6 + ~x,
-            5 + ~x - y,
-            0 & x,
-            6 + (x |= 0),
-        ].forEach(function(n) {
-            console.log(typeof n, n);
-        });
+        function f(num) {
+            var x = "" + num, y = null;
+            [
+                x + "12",
+                2 * x,
+                +x + 3,
+                1 + x + "23",
+                3 | x,
+                6 + x--,
+                x*y + 6,
+                6 + x,
+                6 + ~x,
+                5 + ~x,
+                0 & x,
+                6 + (x |= 0),
+            ].forEach(function(n) {
+                console.log(typeof n, n);
+            });
+        }
+        f(42);
     }
     expect_stdout: [
         "string 4212",
@@ -273,7 +338,7 @@ evaluate_3: {
         console.log(1 + Number(x) + 2);
     }
     expect: {
-        console.log(+x + 3);
+        console.log(+("" + x) + 3);
     }
 }
 
@@ -310,45 +375,52 @@ evaluate_4: {
 evaluate_5: {
     options = {
         evaluate: true,
+        reduce_vars: true,
         unsafe_math: false,
     }
     input: {
-        var a = "1";
-        [
-            +a + 2 + 3,
-            +a + 2 - 3,
-            +a - 2 + 3,
-            +a - 2 - 3,
-            2 + +a + 3,
-            2 + +a - 3,
-            2 - +a + 3,
-            2 - +a - 3,
-            2 + 3 + +a,
-            2 + 3 - +a,
-            2 - 3 + +a,
-            2 - 3 - +a,
-        ].forEach(function(n) {
-            console.log(typeof n, n);
-        });
+        function f(num) {
+            var a = "" + num;
+            [
+                +a + 2 + 3,
+                +a + 2 - 3,
+                +a - 2 + 3,
+                +a - 2 - 3,
+                2 + +a + 3,
+                2 + +a - 3,
+                2 - +a + 3,
+                2 - +a - 3,
+                2 + 3 + +a,
+                2 + 3 - +a,
+                2 - 3 + +a,
+                2 - 3 - +a,
+            ].forEach(function(n) {
+                console.log(typeof n, n);
+            });
+        }
+        f(1);
     }
     expect: {
-        var a = "1";
-        [
-            +a + 2 + 3,
-            +a + 2 - 3,
-            a - 2 + 3,
-            a - 2 - 3,
-            +a + 2 + 3,
-            +a + 2 - 3,
-            2 - a + 3,
-            2 - a - 3,
-            +a + 5,
-            5 - a,
-            +a - 1,
-            -1 - a,
-        ].forEach(function(n) {
-            console.log(typeof n, n);
-        });
+        function f(num) {
+            var a = "" + num;
+            [
+                +a + 2 + 3,
+                +a + 2 - 3,
+                a - 2 + 3,
+                a - 2 - 3,
+                +a + 2 + 3,
+                +a + 2 - 3,
+                2 - a + 3,
+                2 - a - 3,
+                +a + 5,
+                5 - a,
+                +a - 1,
+                -1 - a,
+            ].forEach(function(n) {
+                console.log(typeof n, n);
+            });
+        }
+        f(1);
     }
     expect_stdout: [
         "number 6",
@@ -369,45 +441,52 @@ evaluate_5: {
 evaluate_5_unsafe_math: {
     options = {
         evaluate: true,
+        reduce_vars: true,
         unsafe_math: true,
     }
     input: {
-        var a = "1";
-        [
-            +a + 2 + 3,
-            +a + 2 - 3,
-            +a - 2 + 3,
-            +a - 2 - 3,
-            2 + +a + 3,
-            2 + +a - 3,
-            2 - +a + 3,
-            2 - +a - 3,
-            2 + 3 + +a,
-            2 + 3 - +a,
-            2 - 3 + +a,
-            2 - 3 - +a,
-        ].forEach(function(n) {
-            console.log(typeof n, n);
-        });
+        function f(num) {
+            var a = "" + num;
+            [
+                +a + 2 + 3,
+                +a + 2 - 3,
+                +a - 2 + 3,
+                +a - 2 - 3,
+                2 + +a + 3,
+                2 + +a - 3,
+                2 - +a + 3,
+                2 - +a - 3,
+                2 + 3 + +a,
+                2 + 3 - +a,
+                2 - 3 + +a,
+                2 - 3 - +a,
+            ].forEach(function(n) {
+                console.log(typeof n, n);
+            });
+        }
+        f(1);
     }
     expect: {
-        var a = "1";
-        [
-            +a + 5,
-            +a + -1,
-            a - -1,
-            a - 5,
-            +a + 5,
-            +a + -1,
-            5 - a,
-            -1 - a,
-            +a + 5,
-            5 - a,
-            +a - 1,
-            -1 - a,
-        ].forEach(function(n) {
-            console.log(typeof n, n);
-        });
+        function f(num) {
+            var a = "" + num;
+            [
+                +a + 5,
+                +a + -1,
+                a - -1,
+                a - 5,
+                +a + 5,
+                +a + -1,
+                5 - a,
+                -1 - a,
+                +a + 5,
+                5 - a,
+                +a - 1,
+                -1 - a,
+            ].forEach(function(n) {
+                console.log(typeof n, n);
+            });
+        }
+        f(1);
     }
     expect_stdout: [
         "number 6",
@@ -546,37 +625,44 @@ evaluate_6_unsafe_math: {
 evaluate_7: {
     options = {
         evaluate: true,
+        reduce_vars: true,
         unsafe_math: false,
     }
     input: {
-        var x = "42", y;
-        [
-            +x + 2 + (3 + !y),
-            +x + 2 + (3 - !y),
-            +x + 2 - (3 + !y),
-            +x + 2 - (3 - !y),
-            +x - 2 + (3 + !y),
-            +x - 2 + (3 - !y),
-            +x - 2 - (3 + !y),
-            +x - 2 - (3 - !y),
-        ].forEach(function(n) {
-            console.log(typeof n, n);
-        });
+        function f(num, y) {
+            var x = "" + num;
+            [
+                +x + 2 + (3 + !y),
+                +x + 2 + (3 - !y),
+                +x + 2 - (3 + !y),
+                +x + 2 - (3 - !y),
+                +x - 2 + (3 + !y),
+                +x - 2 + (3 - !y),
+                +x - 2 - (3 + !y),
+                +x - 2 - (3 - !y),
+            ].forEach(function(n) {
+                console.log(typeof n, n);
+            });
+        }
+        f(42);
     }
     expect: {
-        var x = "42", y;
-        [
-            +x + 2 + (3 + !y),
-            +x + 2 + (3 - !y),
-            +x + 2 - (3 + !y),
-            +x + 2 - (3 - !y),
-            x - 2 + (3 + !y),
-            x - 2 + (3 - !y),
-            x - 2 - (3 + !y),
-            x - 2 - (3 - !y),
-        ].forEach(function(n) {
-            console.log(typeof n, n);
-        });
+        function f(num, y) {
+            var x = "" + num;
+            [
+                +x + 2 + (3 + !y),
+                +x + 2 + (3 - !y),
+                +x + 2 - (3 + !y),
+                +x + 2 - (3 - !y),
+                x - 2 + (3 + !y),
+                x - 2 + (3 - !y),
+                x - 2 - (3 + !y),
+                x - 2 - (3 - !y),
+            ].forEach(function(n) {
+                console.log(typeof n, n);
+            });
+        }
+        f(42);
     }
     expect_stdout: [
         "number 48",
@@ -593,37 +679,44 @@ evaluate_7: {
 evaluate_7_unsafe_math: {
     options = {
         evaluate: true,
+        reduce_vars: true,
         unsafe_math: true,
     }
     input: {
-        var x = "42", y;
-        [
-            +x + 2 + (3 + !y),
-            +x + 2 + (3 - !y),
-            +x + 2 - (3 + !y),
-            +x + 2 - (3 - !y),
-            +x - 2 + (3 + !y),
-            +x - 2 + (3 - !y),
-            +x - 2 - (3 + !y),
-            +x - 2 - (3 - !y),
-        ].forEach(function(n) {
-            console.log(typeof n, n);
-        });
+        function f(num, y) {
+            var x = "" + num;
+            [
+                +x + 2 + (3 + !y),
+                +x + 2 + (3 - !y),
+                +x + 2 - (3 + !y),
+                +x + 2 - (3 - !y),
+                +x - 2 + (3 + !y),
+                +x - 2 + (3 - !y),
+                +x - 2 - (3 + !y),
+                +x - 2 - (3 - !y),
+            ].forEach(function(n) {
+                console.log(typeof n, n);
+            });
+        }
+        f(42);
     }
     expect: {
-        var x = "42", y;
-        [
-            +x + 5 + !y,
-            +x + 5 - !y,
-            +x + -1 - !y,
-            +x + -1 + !y,
-            x - -1 + !y,
-            x - -1 - !y,
-            x - 5 - !y,
-            x - 5 + !y,
-        ].forEach(function(n) {
-            console.log(typeof n, n);
-        });
+        function f(num, y) {
+            var x = "" + num;
+            [
+                +x + 5 + !y,
+                +x + 5 - !y,
+                +x + -1 - !y,
+                +x + -1 + !y,
+                x - -1 + !y,
+                x - -1 - !y,
+                x - 5 - !y,
+                x - 5 + !y,
+            ].forEach(function(n) {
+                console.log(typeof n, n);
+            });
+        }
+        f(42);
     }
     expect_stdout: [
         "number 48",
@@ -637,6 +730,22 @@ evaluate_7_unsafe_math: {
     ]
 }
 
+evaluate_8_unsafe_math: {
+    options = {
+        evaluate: true,
+        unsafe_math: true,
+    }
+    input: {
+        var a = [ "42" ];
+        console.log(a * (1 / 7));
+    }
+    expect: {
+        var a = [ "42" ];
+        console.log(+a / 7);
+    }
+    expect_stdout: "6"
+}
+
 NaN_redefined: {
     options = {
         evaluate: true,
@@ -1251,3 +1360,101 @@ issue_3695: {
     }
     expect_stdout: "NaN"
 }
+
+issue_4137: {
+    options = {
+        evaluate: true,
+    }
+    input: {
+        console.log(+(A = []) * (A[0] = 1));
+    }
+    expect: {
+        console.log(+(A = []) * (A[0] = 1));
+    }
+    expect_stdout: "0"
+}
+
+issue_4142: {
+    options = {
+        evaluate: true,
+    }
+    input: {
+        console.log("" + +(0 === console));
+    }
+    expect: {
+        console.log("" + +(0 === console));
+    }
+    expect_stdout: "0"
+}
+
+issue_4542_1: {
+    options = {
+        evaluate: true,
+        unsafe_math: true,
+    }
+    input: {
+        console.log(function(a) {
+            return a / (1 / (a[0] = 2));
+        }([ 3 ]));
+    }
+    expect: {
+        console.log(function(a) {
+            return a / (1 / (a[0] = 2));
+        }([ 3 ]));
+    }
+    expect_stdout: "4"
+}
+
+issue_4542_2: {
+    options = {
+        evaluate: true,
+        unsafe_math: true,
+    }
+    input: {
+        console.log(function(a) {
+            return a / (1 / --a[0]);
+        }([ 3 ]));
+    }
+    expect: {
+        console.log(function(a) {
+            return a / (1 / --a[0]);
+        }([ 3 ]));
+    }
+    expect_stdout: "4"
+}
+
+issue_4542_3: {
+    options = {
+        evaluate: true,
+        unsafe_math: true,
+    }
+    input: {
+        console.log(function(a) {
+            return a / (0 / (a[0] = 0, 1));
+        }([ 1 ]));
+    }
+    expect: {
+        console.log(function(a) {
+            return a / (0 / (a[0] = 0, 1));
+        }([ 1 ]));
+    }
+    expect_stdout: "NaN"
+}
+
+issue_4542_4: {
+    options = {
+        evaluate: true,
+        unsafe_math: true,
+    }
+    input: {
+        console.log(function(a) {
+            return a / (1 / (a.length = 1));
+        }([ 2, 3 ]));
+    }
+    expect: {
+        console.log(function(a) {
+            return a / (1 / (a.length = 1));
+        }([ 2, 3 ]));
+    }
+    expect_stdout: "2"
+}
