@@ -1,6 +1,7 @@
 exports["Compressor"] = Compressor;
 exports["defaults"] = defaults;
 exports["JS_Parse_Error"] = JS_Parse_Error;
+exports["List"] = List;
 exports["mangle_properties"] = mangle_properties;
 exports["minify"] = minify;
 exports["OutputStream"] = OutputStream;
@@ -12,3 +13,4 @@ exports["to_ascii"] = to_ascii;
 exports["tokenizer"] = tokenizer;
 exports["TreeTransformer"] = TreeTransformer;
 exports["TreeWalker"] = TreeWalker;
+exports["vlq_decode"] = vlq_decode;
