@@ -9,6 +9,6 @@ require("./run")([
     "-mc keep_fargs=false,passes=3,pure_getters,unsafe,unsafe_comps,unsafe_math,unsafe_proto",
 ].map(function(options) {
     var args = options.split(/ /);
-    args.unshift("test/benchmark.js");
+    args.unshift("test/benchmark.js", "--validate");
     return args;
 }));
